@@ -1,12 +1,22 @@
 from abc import ABC
 from dataclasses import dataclass
-from typing import Any, AsyncIterable, Dict, List, Set, Tuple
+from typing import (
+    Any,
+    AsyncIterable,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Set,
+    Tuple,
+)
 from uuid import UUID, uuid4
 
 import numpy as np
 import torch
 from aiostream import stream
 from kilroy_module_server_py_sdk import (
+    CategorizableBasedOptionalParameter,
     CategorizableBasedParameter,
     JSONSchema,
     Metric,
@@ -24,6 +34,7 @@ from kilroy_module_pytorch_py_sdk.codec import Codec
 from kilroy_module_pytorch_py_sdk.generator import Generator
 from kilroy_module_pytorch_py_sdk.models import LanguageModel
 from kilroy_module_pytorch_py_sdk.optimizers import Optimizer
+from kilroy_module_pytorch_py_sdk.schedulers.base import Scheduler
 from kilroy_module_pytorch_py_sdk.tokenizer import Tokenizer
 from kilroy_module_pytorch_py_sdk.utils import (
     pack_list,
@@ -69,30 +80,58 @@ class ReinforcedScoreMetric(Metric[Dict]):
         }
 
 
+@dataclass
+class MetricsState:
+    supervised_loss_metric: SupervisedLossMetric
+    reinforced_score_metric: ReinforcedScoreMetric
+
+
+@dataclass
+class ReportsState:
+    epoch_supervised_losses: List[float]
+    epoch_reinforced_scores: List[float]
+
+
 @dataclass
 class State:
     model: LanguageModel
     tokenizer: Tokenizer
     optimizer: Optimizer
     optimizers_params: Dict[str, Dict[str, Any]]
+    scheduler: Optional[Scheduler]
+    schedulers_params: Dict[str, Dict[str, Any]]
     generator: Generator
     codec: Codec
     results_cache: Dict[UUID, Tuple[Tensor, Tensor]]
     batch_size: int
     epoch: int
-    supervised_loss_metric: SupervisedLossMetric
-    reinforced_score_metric: ReinforcedScoreMetric
-    epoch_supervised_losses: List[float]
-    epoch_reinforced_scores: List[float]
+    metrics: MetricsState
+    reports: ReportsState
 
 
 class OptimizerParameter(CategorizableBasedParameter[State, Optimizer]):
     async def _get_params(self, state: State, category: str) -> Dict[str, Any]:
         return {
-            "params": state.model.parameters(),
+            "parameters": state.model.parameters(),
             **state.optimizers_params.get(category, {}),
         }
 
+    async def _set_categorizable(self, state: State, value: Optimizer) -> None:
+        await super()._set_categorizable(state, value)
+        if state.scheduler is not None:
+            optimizer = await value.get()
+            await state.scheduler.change_optimizer(optimizer)
+
+
+class SchedulerParameter(
+    CategorizableBasedOptionalParameter[State, Scheduler]
+):
+    async def _get_params(self, state: State, category: str) -> Dict[str, Any]:
+        return {
+            "optimizer": await state.optimizer.get(),
+            **state.schedulers_params.get(category, {}),
+        }
+
 
 class GeneratorParameter(NestedParameter[State, Generator]):
     pass
@@ -117,6 +156,7 @@ class BasicModule(Module[State], ABC):
     def parameters(cls) -> Set[Parameter]:
         return {
             OptimizerParameter(),
+            SchedulerParameter(),
             GeneratorParameter(),
             CodecParameter(),
             BatchSizeParameter(),
@@ -125,8 +165,8 @@ class BasicModule(Module[State], ABC):
     async def get_metrics(self) -> Set[Metric]:
         async with self.state.read_lock() as state:
             return {
-                state.supervised_loss_metric,
-                state.reinforced_score_metric,
+                state.metrics.supervised_loss_metric,
+                state.metrics.reinforced_score_metric,
             }
 
     async def generate(
@@ -165,7 +205,7 @@ class BasicModule(Module[State], ABC):
             async for batch in streamer:
                 async with self.state.write_lock() as state:
                     loss = await background(fit, state.model, batch)
-                    state.epoch_supervised_losses.append(loss)
+                    state.reports.epoch_supervised_losses.append(loss)
 
     async def fit_posts(
         self, posts: AsyncIterable[Tuple[Dict[str, Any], float]]
@@ -193,7 +233,7 @@ class BasicModule(Module[State], ABC):
 
         async with self.state.write_lock() as state:
             score = await background(fit)
-            state.epoch_reinforced_scores.append(score)
+            state.reports.epoch_reinforced_scores.append(score)
 
     async def fit_scores(self, scores: List[Tuple[UUID, float]]) -> None:
         async def get_results():
@@ -205,23 +245,35 @@ class BasicModule(Module[State], ABC):
 
         await self._fit_reinforced(get_results())
 
+    @staticmethod
+    async def _report_mean_from_epoch(
+        metric: Metric, epoch: int, label: str, values: Iterable[float]
+    ) -> None:
+        values = list(values)
+        if values:
+            await metric.report({"epoch": epoch, label: np.mean(values)})
+
+    @staticmethod
+    async def _reset_reports(state: State) -> None:
+        state.reports.epoch_supervised_losses = []
+        state.reports.epoch_reinforced_scores = []
+
     async def step(self) -> None:
         async with self.state.write_lock() as state:
             await state.optimizer.step()
-            if state.epoch_supervised_losses:
-                await state.supervised_loss_metric.report(
-                    {
-                        "epoch": state.epoch,
-                        "loss": np.mean(state.epoch_supervised_losses),
-                    }
-                )
-            if state.epoch_reinforced_scores:
-                await state.reinforced_score_metric.report(
-                    {
-                        "epoch": state.epoch,
-                        "score": np.mean(state.epoch_reinforced_scores),
-                    }
-                )
-            state.epoch_supervised_losses = []
-            state.epoch_reinforced_scores = []
+            if state.scheduler is not None:
+                await state.scheduler.step()
+            await self._report_mean_from_epoch(
+                state.metrics.supervised_loss_metric,
+                state.epoch,
+                "loss",
+                state.reports.epoch_supervised_losses,
+            )
+            await self._report_mean_from_epoch(
+                state.metrics.reinforced_score_metric,
+                state.epoch,
+                "score",
+                state.reports.epoch_reinforced_scores,
+            )
+            await self._reset_reports(state)
             state.epoch += 1
