@@ -1,19 +1,13 @@
-from dataclasses import dataclass
-from pathlib import Path
-from typing import Any, Dict, List
+from typing import Any, Dict, Iterable, List
 
-import torch
-from kilroy_module_server_py_sdk import (
-    Configurable,
-    SerializableModel,
-    background,
-    classproperty,
-)
+from kilroy_module_server_py_sdk import SerializableModel, classproperty
+from torch import Tensor
 from torch.optim import Adam
 
 from kilroy_module_pytorch_py_sdk.optimizers.base import (
-    Optimizer,
     OptimizerParameter,
+    StandardOptimizer,
+    StandardOptimizerState as State,
 )
 
 
@@ -24,12 +18,7 @@ class Params(SerializableModel):
     weight_decay: float = 0
 
 
-@dataclass
-class State:
-    optimizer: Adam
-
-
-class AdamOptimizer(Optimizer, Configurable[State]):
+class AdamOptimizer(StandardOptimizer):
     class LrParameter(OptimizerParameter[State, float]):
         @classproperty
         def schema(cls) -> Dict[str, Any]:
@@ -67,28 +56,8 @@ class AdamOptimizer(Optimizer, Configurable[State]):
         def schema(cls) -> Dict[str, Any]:
             return {"type": "number", "minimum": 0}
 
-    async def build_default_state(self) -> State:
-        model_params = self._kwargs.pop("parameters")
+    async def _build_default_optimizer(
+        self, parameters: Iterable[Tensor]
+    ) -> Adam:
         user_params = Params(**self._kwargs)
-        return State(optimizer=Adam(model_params, **user_params.dict()))
-
-    @classmethod
-    async def save_state(cls, state: State, directory: Path) -> None:
-        with open(directory / "optimizer.pt", "wb") as f:
-            await background(torch.save, state.optimizer.state_dict(), f)
-
-    async def load_saved_state(self, directory: Path) -> State:
-        with open(directory / "optimizer.pt", "rb") as f:
-            state_dict = await background(torch.load, f)
-        optimizer = Adam(self._kwargs.pop("parameters"))
-        optimizer.load_state_dict(state_dict)
-        return State(optimizer=optimizer)
-
-    async def step(self) -> None:
-        async with self.state.write_lock() as state:
-
-            def step():
-                state.optimizer.step()
-                state.optimizer.zero_grad()
-
-            await background(step)
+        return Adam(parameters, **user_params.dict())
